#!/usr/bin/perl
# CGI-script to run LaTeX, dvips, ps2ps, ps2pdf etc.
#
# $Id: printout.pl,v 1.40.2.2 2004/03/04 05:21:34 albertel Exp $
#
# Copyright Michigan State University Board of Trustees
#
# This file is part of the LearningOnline Network with CAPA (LON-CAPA).
#
# LON-CAPA is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# LON-CAPA is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with LON-CAPA; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# /home/httpd/html/adm/gpl.txt
#
# http://www.lon-capa.org/
#

use lib '/home/httpd/lib/perl';
use LONCAPA::loncgi();
use File::Path;
use IO::File;
use Image::Magick;

my %origENV=%ENV;
if (! &LONCAPA::loncgi::check_cookie_and_load_env()) {
    print <<END;
Content-type: text/html

<html>
<head><title>Bad Cookie</title></head>
<body>
Your cookie information is incorrect.
</body>
</html>
END
    return;
}

 print "Content-type: text/html\n\n";
 print "<body bgcolor=\"#FFFFFF\">\n";

  my $identifier = $ENV{'QUERY_STRING'};
  my $texfile = $ENV{'cgi.'.$identifier.'.file'};
  my $laystyle = $ENV{'cgi.'.$identifier.'.layout'};
  my $numberofcolumns = $ENV{'cgi.'.$identifier.'.numcol'};
  my $selectionmade = $ENV{'cgi.'.$identifier.'.selection'};
  my $tableofcontents = $ENV{'cgi.'.$identifier.'tableofcontents'};
  my $tableofindex = $ENV{'cgi.'.$identifier.'tableofindex'};
  my $advans_role = $ENV{'cgi.'.$identifier.'role'};

  my $figfile = $texfile;
  $figfile =~ s/^([^\.]+printout)[^t]+\.tex/$1\.dat/;
  my $duefile = $texfile;
  $duefile =~ s/^([^\.]+printout)[^t]+\.tex/$1\.due/;
  #do we have figures?
  if (-e $figfile) {
      my $temporary_file=IO::File->new($figfile) || die "Couldn't open file for reading: $!\n";
      my @content_of_file = <$temporary_file>;
      close $temporary_file;  
      my $noteps;
      foreach $not_eps (@content_of_file) {
	  if ($not_eps ne '') {
	      $not_eps=~s|\/\.\/|\/|g;
	      my $eps_f = $not_eps;
	      $eps_f =~ s/\.[^.]*$/\.eps/i;
	      if ($eps_f=~/\/home\/([^\/]+)\/public_html\//) {
                  $eps_f=~s/\/home\/([^\/]+)\/public_html/$1/;
		  $eps_f = '/home/httpd/prtspool/'.$eps_f;
	      } else {
		  $eps_f=~m/\/home\/httpd\/html\/res\/(.+)/;
		  $eps_f = '/home/httpd/prtspool/'.$1;
	      }
	      my $path=$eps_f;
	      $path=~s/\/([^\/]+)\.eps$//;
	      File::Path::mkpath($path,0,0777);
	      my $image = Image::Magick->new;
	      $not_eps =~ s/^\s+//;
	      $not_eps =~ s/\s+$//;
	      $status = $image->Read($not_eps);
	      if ($status) {print "  $status  ";}
	      $image->Set(page => '+100+200'); 
	      $status = $image->Write($eps_f);	    
	      if ($status) {print "  $status  ";}
              #check is eps exist in prtspool
              if(not -e $eps_f) {
		  for (my $i=0;$i<10000;$i++) {
		      if (-e $eps_f.'.'.$i) {
			  rename $eps_f.'.'.$i, $eps_f;
			  last;
		      }
		  }
	      }  
	  }
      }
  }
  #print "$texfile\n"; #name of the tex file for debugging only
  if (-e $texfile) {
      $texfile =~ m/^(.*)\/([^\/]+)$/; 
      my $name_file = $2;
      my $path_file = $1.'/';
      chdir $path_file;
      system("latex $name_file 1>/dev/null 2>/dev/null");
      if ($tableofcontents eq 'yes') {
	  system("latex $name_file 1>/dev/null 2>/dev/null");
      } #to create table of contents
      my $idxname=$name_file;
      $idxname=~s/\.tex$/\.idx/;
      if ($tableofindex eq 'yes') {
	  system("makeindex $idxname");
	  system("latex $name_file 1>/dev/null 2>/dev/null");
      } #to create index
      #Do we have a latex error in the log file?
      my $logfilename = $texfile;
      $logfilename =~ s/\.tex$/\.log/;
      my $temporary_file=IO::File->new($logfilename) || die "Couldn't open file for reading: $!\n";
      my @content_of_file = <$temporary_file>;
      close $temporary_file; 
      my $body_log_file = join(' ',@content_of_file);
      $logfilename =~ s/\.log$/\.html/;
      $temporary_file = IO::File->new('>'.$logfilename); 
      print $temporary_file '<html><head><title>LOGFILE</title></head><body><pre>'.$body_log_file.'</pre></body></html>'."\n";
      if ($body_log_file=~m/!\s+Emergency stop/) {
	  #LaTeX failed to parse tex file 
	  print "<h2>LaTeX could not successfully parse your tex file.</h2>";
	  print "It probably has errors in it.<br />";
	  my $whereitbegins = rindex $body_log_file,'STAMPOFPASSEDRESOURCESTART';
	  my $whereitends = rindex $body_log_file,'STAMPOFPASSEDRESOURCEEND';
	  if ($whereitbegins!=-1 and $whereitends!=-1) {
	      print "With very high probability this error occured in ".substr($body_log_file,$whereitbegins+26,$whereitends-$whereitbegins-26)."<br /><br />";
	  }
	  print "Here are the error messages in the LaTeX log file</br><br />";
	  my $sygnal = 0;
	  for (my $i=0;$i<=$#content_of_file;$i++) {
	      if ($content_of_file[$i]=~m/^Runaway argument?/ or $content_of_file[$i]=~m/^!/) {
		  $sygnal = 1;
	      } 
	      if ($content_of_file[$i]=~m/Here is how much of/) {
		  $sygnal = 0;
	      } 
	      if ($sygnal) {
		  print "$content_of_file[$i]<br />";
	      }  
	  }
          if ($advans_role) {  
              print "<b><big>The link to ";
              $logfilename=~s/\/home\/httpd//;
	      print "<a href=\"$logfilename\">Your log file </a></big></b>";
	      print "\n";
              #link tooriginal LaTeX file (included according Michael Hamlin desire)
	      my $tex_temporary_file=IO::File->new($texfile) || die "Couldn't open file for reading: $!\n";
	      my @tex_content_of_file = <$tex_temporary_file>;
	      close $tex_temporary_file; 
	      my $body_tex_file = join(' ',@tex_content_of_file);
	      $texfile =~ s/\.tex$/aaaaa\.html/;
	      $tex_temporary_file = IO::File->new('>'.$texfile); 
	      print $tex_temporary_file '<html><head><title>LOGFILE</title></head><body><pre>'.$body_tex_file.'</pre></body></html>'."\n";
	      print "<br /><br />";
	      print "<b><big>The link to ";
	      $texfile=~s/\/home\/httpd//;
	      print "<a href=\"$texfile\">Your original LaTeX file </a></big></b>";
	      print "\n";
	  }

      } elsif ($body_log_file=~m/<inserted text>/) {
	  my $whereitbegins = index $body_log_file,'<inserted text>';
	  print "You are running LaTeX in the <b>batch mode</b>.";
	  while ($whereitbegins != -1) {
	      my $tempobegin=$whereitbegins;
	      $whereitbegins = rindex $body_log_file,'STAMPOFPASSEDRESOURCESTART',$whereitbegins;
	      my $whereitends = index $body_log_file,'STAMPOFPASSEDRESOURCEEND',$whereitbegins;
	      print "<br />It has found an error in".substr($body_log_file,$whereitbegins+26,$whereitends-$whereitbegins-26)." <br /> and corrected it.\n";
	      print "Usually this correction is valid but you probably need to check the indicated resource one more time and implement neccessary corrections by yourself.\n";
	      $whereitbegins = index $body_log_file,'<inserted text>',$tempobegin+10;
	  }
	  $name_file =~ s/\.tex/\.dvi/;
	  my $new_name_file = $name_file;
	  $new_name_file =~ s/\.dvi/\.ps/;
	  my $comma = "dvips -Ppdf -G0 -o $new_name_file";
	  system("$comma $name_file 1>/dev/null 2>/dev/null");
	  if (-e $new_name_file) {
	      print "<h1>PDF output file (see link below)</h1>\n";
	      $new_name_file =~ m/^(.*)\./;
	      my $tempo_file = $1.'temporar.ps';
	      my $name_file = $1.'.pdf';
	      if ($laystyle eq 'album' and $numberofcolumns eq '2') {
		  $comma = "psnup -2 -s1.0 $new_name_file";
		  system("$comma $tempo_file 1>/dev/null 2>/dev/null"); 
		  system("ps2pdf $tempo_file $name_file 1>/dev/null 2>/dev/null");
	      } elsif ($laystyle eq 'book' and $numberofcolumns eq '2') {
		  $comma = 'pstops -pletter "2:0+1(0.48w,0)"';
		  system("$comma $new_name_file $tempo_file 1>/dev/null 2>/dev/null");
		  system("ps2pdf $tempo_file $name_file 1>/dev/null 2>/dev/null");
	      } else {
		  system("ps2pdf $new_name_file $name_file 1>/dev/null 2>/dev/null");
	      }	    
	      my $texlog = $texfile;
	      my $texaux = $texfile;
	      my $texdvi = $texfile;
	      my $texps = $texfile;
	      $texlog =~ s/\.tex/\.log/;
	      $texaux =~ s/\.tex/\.aux/;
	      $texdvi =~ s/\.tex/\.dvi/;
	      $texps =~ s/\.tex/\.ps/;
	      my @garb = ($texaux,$texdvi,$texps);
#	  unlink @garb;
	      unlink $duefile;
	      print "<a href=\"/prtspool/$name_file\">Your PDF document</a>";
	      if ($advans_role) {  
		  print "<br /><br />";
		  print "<b><big>The link to ";
		  $logfilename=~s/\/home\/httpd//;
		  print "<a href=\"$logfilename\">Your log file </a></big></b>";
		  print "\n";
		  #link tooriginal LaTeX file (included according Michael Hamlin desire)
		  my $tex_temporary_file=IO::File->new($texfile) || die "Couldn't open file for reading: $!\n";
		  my @tex_content_of_file = <$tex_temporary_file>;
		  close $tex_temporary_file; 
		  my $body_tex_file = join(' ',@tex_content_of_file);
		  $texfile =~ s/\.tex$/aaaaa\.html/;
		  $tex_temporary_file = IO::File->new('>'.$texfile); 
		  print $tex_temporary_file '<html><head><title>LOGFILE</title></head><body><pre>'.$body_tex_file.'</pre></body></html>'."\n";
		  print "<br /><br />";
		  print "<b><big>The link to ";
		  $texfile=~s/\/home\/httpd//;
		  print "<a href=\"$texfile\">Your original LaTeX file </a></big></b>";
		  print "\n";
	      }
	  }
      } else {
	  #LaTeX successfully parsed tex file 
	  $name_file =~ s/\.tex/\.dvi/;
	  my $new_name_file = $name_file;
	  $new_name_file =~ s/\.dvi/\.ps/;
	  my $comma = "dvips -Ppdf -G0 -o $new_name_file";
	  system("$comma $name_file 1>/dev/null 2>/dev/null");
	  if (-e $new_name_file) {
	      print "<h1>Successfully created PDF output file (see link below)</h1>\n";
	      $new_name_file =~ m/^(.*)\./;
	      my $tempo_file = $1.'temporar.ps';
	      my $name_file = $1.'.pdf';
	      if ($laystyle eq 'album' and $numberofcolumns eq '2') {
		  $comma = "psnup -2 -s1.0 $new_name_file";
		  system("$comma $tempo_file 1>/dev/null 2>/dev/null"); 
		  system("ps2pdf $tempo_file $name_file 1>/dev/null 2>/dev/null");
	      } elsif ($laystyle eq 'book' and $numberofcolumns eq '2') {
		  $comma = 'pstops -pletter "2:0+1(0.48w,0)"';
		  system("$comma $new_name_file $tempo_file 1>/dev/null 2>/dev/null");
		  system("ps2pdf $tempo_file $name_file 1>/dev/null 2>/dev/null");
	      } else {
		  system("ps2pdf $new_name_file $name_file 1>/dev/null 2>/dev/null");
	      }	    
	      my $texlog = $texfile;
	      my $texaux = $texfile;
	      my $texdvi = $texfile;
	      my $texps = $texfile;
	      $texlog =~ s/\.tex/\.log/;
	      $texaux =~ s/\.tex/\.aux/;
	      $texdvi =~ s/\.tex/\.dvi/;
	      $texps =~ s/\.tex/\.ps/;
	      my @garb = ($texlog,$texaux,$texdvi,$texps);
#	  unlink @garb;
	      unlink $duefile;
	      print "<a href=\"/prtspool/$name_file\">Your PDF document</a>";
	      print "\n";
	  }
      }
  } else {
      print "LaTeX file $texfile was not created successfully";
  }
  #got what we needed reset ENV in case it is to big for system
  %ENV=%origENV;


  





